@@ -1,21 +1,30 @@
-#  PRESENTATION PLOTS
+#  PRESENTATION PLOTS FROM FITTED MODEL OBJECTS
 
-volcanoplot <- function(fit, coef=1, highlight=0, names=fit$genes$ID, ...)
-#	Volcano plot of log-fold-change and B-statistic
+volcanoplot <- function(fit,coef=1L,style="p-value",highlight=0L,names=fit$genes$ID,hl.col="blue",xlab="Log2 Fold Change",ylab=NULL,pch=16,cex=0.35, ...)
+#	Volcano plot of log-fold-change vs significance (p-value or B-statistic)
 #	Gordon Smyth
-#	27 Oct 2006.
+#	Created 27 Oct 2006. Last modified 14 Oct 2020.
 {
 	if(!is(fit,"MArrayLM")) stop("fit must be an MArrayLM")
-	if(is.null(fit$lods)) stop("No B-statistics found, perhaps eBayes() not yet run")
 	x <- as.matrix(fit$coef)[,coef]
-	y <- as.matrix(fit$lods)[,coef]
-	plot(x,y,xlab="Log Fold Change",ylab="Log Odds",pch=16,cex=0.2,...)
+	style <- match.arg(tolower(style), c("p-value","b-statistic"))
+	if(style=="p-value") {
+		if(is.null(fit$p.value)) stop("No p-values found in linear model fit object. Please run eBayes.")
+		y <- as.matrix(fit$p.value)[,coef]
+		y <- -log10(y)
+		if(is.null(ylab)) ylab="-log10(P-value)"
+	} else {
+		if(is.null(fit$lods)) stop("No B-statistics found in linear model fit object. Please run eBayes.")
+		y <- as.matrix(fit$lods)[,coef]
+		if(is.null(ylab)) ylab="Log Odds of Differential Expression"
+	}
+	plot(x,y,xlab=xlab,ylab=ylab,pch=pch,cex=cex,...)
 	if(highlight>0) {
 		if(is.null(names)) names <- 1:length(x)
 		names <- as.character(names)
 		o <- order(y,decreasing=TRUE)
 		i <- o[1:highlight]
-		text(x[i],y[i],labels=substring(names[i],1,8),cex=0.8,col="blue")
+		text(x[i],y[i],labels=substring(names[i],1,8),cex=0.8,col=hl.col)
 	}
 	invisible()
 }
@@ -23,7 +32,7 @@ volcanoplot <- function(fit, coef=1, highlight=0, names=fit$genes$ID, ...)
 heatDiagram <- function(results,coef,primary=1,names=NULL,treatments=colnames(coef),limit=NULL,orientation="landscape",low="green",high="red",cex=1,mar=NULL,ncolors=123,...) {
 #	Heat diagram to display fold changes of genes under different conditions
 #	Gordon Smyth
-#	27 Oct 2002. Last revised 11 Oct 2004.
+#	Created 27 Oct 2002. Last revised 11 Oct 2004.
 
 #	Check input
 	results <- as.matrix(results)
@@ -111,7 +120,7 @@ heatdiagram <- function(stat,coef,primary=1,names=NULL,treatments=colnames(stat)
 #	Similar to heatDiagram with classifyTestsT(stat,t1=critical.primary,t2=critical.other)
 #	except that heatdiagram() requires primary column to be significant at the first step-down level
 #	Gordon Smyth
-#	27 Oct 2002. Last revised 25 Feb 2003.  mar added 11 Oct 2004
+#	Created 27 Oct 2002. Argument 'mar' added 11 Oct 2004. Last revised 12 Apr 2020.
 
 #	Check input
 	stat <- as.matrix(stat)
@@ -124,7 +133,7 @@ heatdiagram <- function(stat,coef,primary=1,names=NULL,treatments=colnames(stat)
 
 #  Sort coefficients
 	DE <- (stat[,primary] > critical.primary)
-	if(any(is.na(DE))) DE[is.na(DE)] <- FALSE
+	if(anyNA(DE)) DE[is.na(DE)] <- FALSE
 	ng <- sum(DE)
 	if(sum(DE) == 0) {
 		warning("Nothing significant to plot")
@@ -181,3 +190,63 @@ heatdiagram <- function(stat,coef,primary=1,names=NULL,treatments=colnames(stat)
 	invisible(out)
 }
 
+plotSA <- function(fit, xlab="Average log-expression", ylab="sqrt(sigma)", zero.weights=FALSE, pch=16, cex=0.3, col=c("black","red"),...)
+#	Plot log-residual variance vs intensity
+#	Gordon Smyth
+#	Created 14 Jan 2009. Last modified 9 April 2025.
+{
+#	Check fit and extract components
+	if(!is(fit,"MArrayLM")) stop("fit must be an MArrayLM object")
+	x <- fit$Amean
+	y <- fit$sigma
+	dfg <- fit$df.residual
+	df0 <- fit$df.prior
+	s20 <- fit$s2.prior
+	if(is.null(x)) stop("fit$Amean is NULL")
+	if(is.null(y)) stop("fit$sigma is NULL")
+	if(is.null(dfg)) stop("fit$df.residual is NULL")
+
+#	By default, don't plot non-estimable variances
+	if(zero.weights) {
+#		Avoid NA later from pf()
+		dfg <- pmax(dfg, 1e-6)
+	} else {
+		y[dfg < 1e-6] <- NA
+		if(!is.null(fit$weights)) {
+			allzero <- rowSums(fit$weights>0,na.rm=TRUE) == 0
+			y[allzero] <- NA
+		}
+	}
+
+#	Make vector of background color
+	colv <- rep_len(col[1],nrow(fit))
+
+#	If robust eBayes, identify and highlight outlier variances
+	if(length(df0) > 1L) {
+		df2 <- max(df0)
+		s2 <- y^2 / s20
+		pdn <- pf(s2, df1=dfg, df2=df2)
+		pup <- pf(s2, df1=dfg, df2=df2, lower.tail=FALSE)
+		FDR <- p.adjust(2*pmin(pdn,pup),method="BH")
+		colv[FDR <= 0.5] <- col[2]
+	}
+
+#	Plot residual variances
+	y <- sqrt(y)
+	plot(x,y,xlab=xlab,ylab=ylab,pch=pch,cex=cex,col=colv,...)
+
+#	Add prior variance
+	if(!is.null(s20)) {
+		if(identical(length(s20),1L)) {
+			abline(h=sqrt(sqrt(s20)),col="blue")
+		} else {
+			o <- order(x)
+			lines(x[o],sqrt(sqrt(s20[o])),col="blue")
+		}
+	}
+
+#	If robust eBayes, add legend about outliers
+	if(length(df0) > 1L) legend("topright",legend=c("Normal","Outlier"),col=col,pch=pch)
+
+	invisible(list(x=x,y=y))
+}
